#!/usr/bin/env python
# _*_ coding:utf-8 _*_

'''
 Copyright: 2016-2018 ROS小课堂 www.corvin.cn
 Author: corvin
 Description:
    ROS解析代码,用来通过串口来获取从arduino发来的数据.然后组装成控制移动的twist命令,
    通过话题发布出来.
 History:
    20181024: initial this code.
'''

import os
import rospy
import dynamic_reconfigure.client
from arduino_driver import Arduino
from geometry_msgs.msg import Twist
from serial.serialutil import SerialException


class HandleROS():
    def __init__(self):
        rospy.init_node('ros_handle_node', log_level=rospy.INFO)
        client = dynamic_reconfigure.client.Client("handle_dynamic_reconfig",timeout=30,config_callback=self.callback)

        # Get the name in case it is set in the launch file
        self.name = rospy.get_name()
        rospy.on_shutdown(self.shutdown)

        self.getConfigParam()

        # Connect arduino board
        self.controller = Arduino(self.port, self.baud, self.timeout)
        self.controller.connect()
        rospy.loginfo("Connected to handle on port " + self.port + " at " +
                      str(self.baud) + " baud!" +" cmd_topic name:" + self.cmd_topic)

        while not rospy.is_shutdown():
            dynamic_param  = client.get_configuration(timeout=3)
            self.dynamicReconfig(dynamic_param)

            self.cmd_vel_pub = rospy.Publisher(self.cmd_topic, Twist, queue_size=self.cmd_delay)
            loop_rate = rospy.Rate(self.rate)

            Data = []
            Data = self.controller.recv_int_array()
            if len(Data) == 5:
                map(float, Data)
                #rospy.loginfo("lock_status:" + str(Data[0])+" handle_x:" + str(Data[1]) + " handle_y:"+str(Data[2])
                #              + " handle_z:"+str(Data[3])+" rotate_percent:"+str(Data[4]))

                if (Data[0] == 1):
                  linear_x, angular_z = self.parseSerialData(Data[1], Data[2], Data[3], Data[4])
                  self.sendCmdTopic(linear_x, angular_z)

            loop_rate.sleep()


    def dynamicReconfig(self, paramList):
        self.cmd_topic = paramList['cmd_topic_name']
        self.rate = paramList['cmd_public_rate']
        self.angular_z = paramList['angular_z']
        self.linear_x  = paramList['linear_x']


    def callback(self, config):
        rospy.loginfo("{cmd_topic_name},{cmd_public_rate},{angular_z},{linear_x}".format(**config))


    def parseSerialData(self, handle_x, handle_y, handle_z, rotate_percent):
        if handle_z == 1:
            linear_x = self.linear_x
            linear_x *= (rotate_percent/100.0)
            self.cmd_linear_x = linear_x
        else:
            angular_z = self.angular_z
            angular_z *= (rotate_percent/100.0)
            self.cmd_angular_z = angular_z

        # config angular_z
        angular_z = self.cmd_angular_z
        if handle_x > 700:
            angular_z *= -1
        elif handle_x < 300:
            angular_z *= 1
        else:
            angular_z = 0.0

        # config linear x
        linear_x = self.cmd_linear_x
        if handle_y > 700:
            linear_x *= 1
        elif handle_y < 300:
            linear_x *= -1
        else:
            linear_x = 0.0

        #rospy.loginfo("Send linear_x: " + str(linear_x)+" angular_z:"+str(angular_z))
        return linear_x, angular_z


    def sendCmdTopic(self, linear_x, angular_z):
        '''Send cmd to topic
        '''
        cmd_msg = Twist()
        cmd_msg.linear.x = linear_x
        cmd_msg.linear.y = 0
        cmd_msg.linear.z = 0

        cmd_msg.angular.x = 0
        cmd_msg.angular.y = 0
        cmd_msg.angular.z = angular_z
        self.cmd_vel_pub.publish(cmd_msg)


    def getConfigParam(self):
        '''Get parameter from config file
        '''
        self.port = rospy.get_param("~port", "/dev/ttyACM0")
        self.baud = int(rospy.get_param("~baud", 57600))
        self.timeout = rospy.get_param("~timeout", 0.7)
        self.cmd_topic = rospy.get_param("~cmd_topic", "/cmd_vel")
        self.angular_z = rospy.get_param("~angular_z", 1.0)
        self.linear_x  = rospy.get_param("~linear_x",  1.0)
        self.rate = int(rospy.get_param("~public_rate", 20))
        self.cmd_delay = int(rospy.get_param("~cmd_delay", 1))
        self.cmd_linear_x  = 1.0
        self.cmd_angular_z = 1.0


    def shutdown(self):
        '''Shutdown the handle
        '''
        rospy.logwarn("Shutting down ros handle node...")
        try:
            self.cmd_vel_pub.publish(Twist())
            rospy.sleep(2)
        except:
            pass
        try:
            self.controller.close()
        except:
            pass
        finally:
            rospy.logwarn("Serial port closed")
            os._exit(0)


if __name__ == '__main__':
    try:
        myHandle = HandleROS()
    except SerialException:
        rospy.logerr("Serial exception trying to open port !")
        os._exit(1)

